@@ -5,7 +5,7 @@ The methods for the class ``FinitelyGeneratedSubgroup`` use a number of ancillar
 A word is a string of characters from either a numerical or an alphabetical set
 of letters: ``alphabet_type='123'`` or ``'abc'``.
 
-``alphabet_type='123'``: The positive letters form an interval `[1,r]`. Their inverses (aka
+``alphabet_type='123'``: The positive letters form an interval `[1,r]`. Their inverses (a.k.a.
 negative letters) are the corresponding negative integers. The symmetrized
 alphabet is the union of positive and negative letters (zero is NOT a letter).
 The `\textit{rank}` of a word is the maximal absolute value of a letter occurring in the word.
@@ -36,10 +36,6 @@ We have functions to:
 
 - compute the image of a word in a ``DiGraph``
 
-- compute a spanning tree
-
-- compute a basis specified by a spanning tree
-
 - prune a ``DiGraph``
 
 - cyclically reduce a ``DiGraph``
@@ -75,9 +71,9 @@ from sage.graphs.digraph import DiGraph
 #from sage.sets.disjoint_set import DisjointSet
 from sage.misc.latex import LatexExpr
 #
-from partial_injections import PartialInjection
+from stallings_graphs.partial_injections import PartialInjection
 from stallings_graphs.partial_injections_misc import *
-from about_words import *
+from stallings_graphs.about_words import *
 
 
 def bouquet(list_of_Words, alphabet_type='123', check=False):
@@ -112,6 +108,12 @@ def bouquet(list_of_Words, alphabet_type='123', check=False):
         sage: G
         Looped multi-digraph on 8 vertices
         
+    ::
+
+        sage: GG = bouquet([[-1]])
+        sage: GG
+        Looped multi-digraph on 1 vertex
+        
     """
     if check:
         if not is_valid_list_of_Words(list_of_Words,alphabet_type):
@@ -125,7 +127,10 @@ def bouquet(list_of_Words, alphabet_type='123', check=False):
     for u in new_list_of_Words:
         new_edge_list = []
         if len(u) == 1:
-            new_edge_list.append([0,0,u[0]])
+            if u[0] > 0:
+                new_edge_list.append([0,0,u[0]])
+            else:
+                new_edge_list.append([0,0,inverse_letter(u[0])])
         elif len(u) > 1:
             if u[0] > 0:
                 new_edge_list.append([0,m + 1,u[0]])
@@ -154,9 +159,9 @@ def transitions(digr):
     Return a dictionary of the transitions (edges) of this ``DiGraph``, organized by edge labels.
     
     ``digr`` is expected to be a ``DiGraph``, with labeled edges and with vertex set of
-    the form `range(n)`. The output dictionary maps each edge label to a List of sets:
-    the image of edge label `x` is a list indexed by the vertex set, where
-    the `v`-entry is the set of end vertices of `x`-labeled edges out of `v`, or
+    the form `[0..n]`. The output dictionary maps each edge label to a list of sets:
+    the image of edge label `a` is a list indexed by the vertex set, where
+    the `v`-entry is the set of end vertices of `a`-labeled edges out of `v`, or
     ``None`` if that set is empty.
     
     INPUT:
@@ -197,7 +202,7 @@ def is_deterministic(digr):
     Return whether this ``DiGraph`` is deterministic.
     
     ``digr`` is expected to be a ``DiGraph`` with labeled edges and with vertex set of
-    the form `range(n)`. It is said to be deterministic if for each vertex `v` and each
+    the form `[0..n]`. It is said to be deterministic if for each vertex `v` and each
     label `a`, there is at most one `a`-labeled edge out of `v`.
     
     INPUT:
@@ -216,6 +221,7 @@ def is_deterministic(digr):
         sage: is_deterministic(digr)
         False
     """
+#    all(all(len(x) == 1 for x in L if x != None) for L in transitions(digr).values())
     determin = True
     for L in transitions(digr).values():
         for x in L:
@@ -227,8 +233,8 @@ def transition_function(digr):
     r"""
     Return a dictionary of the transitions (edges) of this ``DiGraph``.
     
-    ``digr`` is expected to be a deterministic ``DiGraph`` (a ``ValueError`` is raised otherwise), with edge labels
-    positive integers. The output dictionary maps each edge labels to a list:
+    ``digr`` is expected to be a deterministic ``DiGraph`` (a ``ValueError`` is raised otherwise),
+    with edge labels positive integers. The output dictionary maps each edge label to a list:
     the image of edge label `a` is a list indexed by the vertex set, where
     the `v`-entry is ``None`` if one cannot read `a` from `v`, and the result of the
     `a`-transition from `v` otherwise.
@@ -268,7 +274,7 @@ def is_folded(G):
     Return whether this ``DiGraph`` is folded (deterministic and co-deterministic).
     
     ``G`` is expected to be a ``DiGraph`` with labeled edges and with vertex set of
-    the form `range(n)`. It is said to be deterministic if for each vertex `v` and each
+    the form `[0..n]`. It is said to be deterministic if for each vertex `v` and each
     label `a`, there is at most one `a`-labeled edge out of `v`; and co-deterministic if
     for each vertex `v` and label `a`, there is at most one `a`-labeled edge into `v`.
     
@@ -297,14 +303,15 @@ def is_folded(G):
 
 def DiGraph_to_list_of_PartialInjection(G):
     r"""
-    Return the list of partial injections (in fact: objects of class ``PartialInjection``) on the set of vertices of this graph.
+    Return the list of partial injections (in fact: objects of class ``PartialInjection``)
+    on the set of vertices of this graph.
     
-    ``G`` is expected to be a folded ``DiGraph`` (a ``ValueError`` is raised otherwise) with edge labels in `[1..r]` and 
-    vertex set `[0..n-1]` (`r` and `n` not given). Folded means that the graph
-    is deterministic and co-deterministic or, equivalently, that every edge label
-    defines a partial injection on the vertex set. The list returned has size `r`, and
-    represents the partial injections (from the class ``PartialInjection``) induced
-    by edge labels `1, 2, \dots, r`, respectively.
+    ``G`` is expected to be a folded ``DiGraph`` (a ``ValueError`` is raised otherwise)
+    with edge labels in `[1..r]` and vertex set equal to `[0..n-1]` (`r` and `n` not given).
+    Folded means that the graph is deterministic and co-deterministic or, equivalently,
+    that every edge label defines a partial injection on the vertex set. The list returned
+    has size `r`, and represents the partial injections (from the class ``PartialInjection``)
+    induced by edge labels `1, 2, \dots, r`, respectively.
     
     INPUT:
 
@@ -327,21 +334,32 @@ def DiGraph_to_list_of_PartialInjection(G):
         [A partial injection of size 10, whose domain has size 4,
          A partial injection of size 10, whose domain has size 5,
          A partial injection of size 10, whose domain has size 3]
+         
+    ::
+        
+        sage: L = [[3,1,-2,-1,-3],[-1,2,-1,-2,1,2],[3,2,-3,-3,1],[5]]
+        sage: G = bouquet(L)
+        sage: GG = NT_fold(G)
+        sage: DiGraph_to_list_of_PartialInjection(GG)
+        [A partial injection of size 10, whose domain has size 4,
+         A partial injection of size 10, whose domain has size 5,
+         A partial injection of size 10, whose domain has size 3,
+         A partial injection of size 10, whose domain has size 0,
+         A partial injection of size 10, whose domain has size 1]
     
     """
     if not is_folded(G):
         raise ValueError('the argument is not a folded DiGraph')
+    EL = set(G.edge_labels())
     if G.edges():
-        r = max(G.edge_labels())
+        r = max(EL)
     else:
         r = 0
-    n = len(G.vertices())
-    pinj = [[] for _ in range(r)]
-    for a in range(r):
-        pinj[a] = [None for _ in range(n)]
-    for (p,q,i) in G.edges():
-        pinj[i - 1][p] = q
-    pinj = [PartialInjection(x) for x in pinj]
+    NV = len(G.vertices())
+    pinj = [[None for _ in range(NV)] for _ in range(r)]
+    for a in EL:
+        pinj[a-1] = transition_function(G)[a]
+    pinj = [PartialInjection(pinj[x]) for x in range(r)]
     return pinj
 
 
@@ -350,14 +368,14 @@ def image_of_word(G,w,qinitial=0,trace=False):
     Return the vertex reached after reading this word in this graph (``None`` if it cannot be read).
     
     ``G`` is expected to be a ``DiGraph`` whose edges are labeled deterministically and
-    codeterministically (folded ``DiGraph``) by a numerical alphabet (typically, `G` is a Stallings
+    codeterministically (folded ``DiGraph``) by a numerical alphabet (typically, ``G`` is a Stallings
     graph) [not verified], ``w`` is a ``Word`` on a numerical alphabet and ``qinitial``
-    is a vertex of `G`. If one can read `w` from `qinitial` in `G`, the output is the vertex
+    is a vertex of ``G``. If one can read ``w`` from ``qinitial`` in ``G``, the output is the vertex
     reached. If one cannot, the output is ``None``. The option ``trace=True`` documents
-    the situation if `w` cannot be read in `G`: the output is the triple
-    (``None``, ``length_read``, ``last_vertex_visited``) where ``length_read`` is the length of the longest
-    prefix `u` of `w` which can be read in `G` starting at ``qinitial`` and ``last_vertex_visited``
-    is the vertex reached after reading `u`.
+    the situation if ``w`` cannot be read in ``G``: the output is the triple
+    (``None``, ``length_read``, ``last_vertex_visited``) where ``length_read`` is the length
+    of the longest prefix ``u`` of ``w`` which can be read in ``G`` starting at ``qinitial`` and
+    ``last_vertex_visited`` is the vertex reached after reading ``u``.
     
     INPUT:
 
@@ -369,17 +387,9 @@ def image_of_word(G,w,qinitial=0,trace=False):
         
         - ``trace`` -- boolean
 
-    OUTPUT: if ``trace=False``
+    OUTPUT: 
 
-        - integer or ``None``
-
-    if ``trace=True``, a triple consisting of
-        
-        - an integer or ``None``
-
-        - an integer
-        
-        - an integer
+        - integer or ``None`` if ``trace=False``; and if ``trace=True``, a triple consisting of three integers or ``None`` and two integers
 
     EXAMPLES ::
 
@@ -499,11 +509,9 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
 
     OUTPUT: If ``certificate`` is set to ``False``:
 
-    - a boolean
-
-    If ``certificate`` is set to ``True``:
-
-    - a tuple of the form ``(False,None)`` or ``(True,sigma)`` where ``sigma`` is the conjugating permutation
+    - a boolean, if ``certificate`` is set to ``False``; and if ``certificate`` is set to ``True``, 
+      a tuple of the form ``(False,None)`` or ``(True,sigma)`` where ``sigma`` is the
+      conjugating permutation
 
     EXAMPLES ::
 
@@ -521,18 +529,18 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
         sage: are_equal_as_rooted_unlabeled(G1,G2)
         False
 
-        ::
+    ::
 
         sage: are_equal_as_rooted_unlabeled(G1,G3)
         True
 
-        ::
+    ::
 
         sage: (b, tau) = are_equal_as_rooted_unlabeled(G1,G3,certificate=True)
         sage: tau
         [0, 1, 2, 5, 3, 4]
 
-        ::
+    ::
 
         sage: H = FinitelyGeneratedSubgroup([])
         sage: G1 = H.stallings_graph()
@@ -542,7 +550,7 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
         sage: (b,tau)
         (True, [0])
         
-        ::
+    ::
         
         sage: H1 = FinitelyGeneratedSubgroup.from_generators(['a','b'],alphabet_type='abc')
         sage: G1 = H1.stallings_graph()
@@ -579,16 +587,21 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
         rH = 0
     if rG != rH:
         if verbose:
-            print 'the ambient free groups of the arguments are different.'        
+            print('the ambient free groups of the arguments are different.')        
         return (False,None) if (certificate == True) else False
     if len(G.vertices()) != len(H.vertices()):
         if verbose:
-            print 'the Stallings graphs of the subgroups have different sizes.'
+            print('the Stallings graphs of the subgroups have different sizes.')
+        return (False,None) if (certificate == True) else False
+    if len(G.edges()) != len(H.edges()):
+        if verbose:
+            print('the Stallings graphs of the subgroups have different numbers of edges.')
         return (False,None) if (certificate == True) else False
     #
-    # Now self and other are FinitelyGeneratedSubgroup, with the same ambient_group_rank
-    # and the same stallings_graph_size. A common reason for not being identical subgroups
-    # is that the partial injections cannot be translated to one another because they
+    # Now self and other are FinitelyGeneratedSubgroup, with the same ambient_group_rank,
+    # the same stallings_graph_size and the same number of edges.
+    # A common reason for not being identical subgroups is that the partial 
+    # injections cannot be translated to one another because they
     # have different orbit profiles (lists of lengths of sequences and of cycles).       
     ambient_rank = rG
     size = len(G.vertices())
@@ -609,7 +622,7 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
     #
     if (c_lengths1 != c_lengths2) or (s_lengths1 != s_lengths2):
         if verbose:
-            print 'the orbit profiles of the partial injections are different.'
+            print('the orbit profiles of the partial injections are different.')
         return (False,None) if (certificate == True) else False
     #
     # now try to build a permutation sigma, fixing 0 (the base vertex),
@@ -665,22 +678,22 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
             if type1[i][x] == 'seq':
                 if type2[i][sigma[x]] == 'cyc':
                     if verbose:
-                        print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                        print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                     return (False,None) if (certificate == True) else False
                 if len(s_list1[i][orbit1[i][x]]) != len(s_list2[i][orbit2[i][sigma[x]]]):
                     if verbose:
-                        print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                        print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                     return (False,None) if (certificate == True) else False
                 if location1[i][x] != location2[i][sigma[x]]:
                     if verbose:
-                        print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                        print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                     return (False,None) if (certificate == True) else False
                 # now these causes for failure are eliminated
                 for y in range(len(s_list1[i][orbit1[i][x]])):
                     if sigma[s_list1[i][orbit1[i][x]][y]] != None:
                         if sigma[s_list1[i][orbit1[i][x]][y]] != s_list2[i][orbit2[i][sigma[x]]][y]:
                             if verbose:
-                                print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                                print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                             return (False,None) if (certificate == True) else False
                     else:
                         sigma[s_list1[i][orbit1[i][x]][y]] = s_list2[i][orbit2[i][sigma[x]]][y]
@@ -689,11 +702,11 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
             if type1[i][x] == 'cyc':
                 if type2[i][sigma[x]] == 'seq':
                     if verbose:
-                        print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                        print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                     return (False,None) if (certificate == True) else False
                 if len(c_list1[i][orbit1[i][x]]) != len(c_list2[i][orbit2[i][sigma[x]]]):
                     if verbose:
-                        print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                        print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                     return (False,None) if (certificate == True) else False
                 if location1[i][x] > location2[i][sigma[x]]:
                     cycle3 = [c_list2[i][orbit2[i][sigma[x]]][z] for z in range(location2[i][sigma[x]] - location1[i][x], len(c_list1[i][orbit1[i][x]]) + location2[i][sigma[x]] - location1[i][x])]
@@ -703,14 +716,14 @@ def are_equal_as_rooted_unlabeled(G, H, certificate=False, verbose=False):
                     if sigma[c_list1[i][orbit1[i][x]][y]] != None:
                         if sigma[c_list1[i][orbit1[i][x]][y]] != cycle3[y]:
                             if verbose:
-                                print 'ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.'
+                                print('ambient group rank, Stallings graph size and orbit profiles are the same, but there is no conjugating permutation.')
                             return (False,None) if (certificate == True) else False
                     else:
                         sigma[c_list1[i][orbit1[i][x]][y]] = cycle3[y]
                         new_domain.add(c_list1[i][orbit1[i][x]][y])
 #
     if verbose:
-        print 'The permutation {} conjugates H1 to H2.'.format(sigma)
+        print('The permutation {} conjugates H1 to H2.'.format(sigma))
     return (True,sigma) if (certificate == True) else True
 
 
@@ -723,9 +736,9 @@ def relabeling(G, R):
     r"""
     Relabels this ``DiGraph`` using the given permutation.
     
-    ``G`` is expected to a ``DiGraph`` with vertices labeled `0,\dots,n-1`. ``R`` is expected to be a permutation
-    of `0,\dots,n-1`. (No verification is made of that fact.) The output ``DiGraph`` is obtained from ``G`` by
-    relabeling the vertices of ``G`` using the permutation ``R``. 
+    ``G`` is expected to a ``DiGraph`` with vertices labeled in `[0..n]`. ``R`` is expected to be
+    a permutation of `[0..n]`. (No verification is made of that fact.) The output ``DiGraph`` is
+    obtained from ``G`` by relabeling the vertices of ``G`` using the permutation ``R``. 
             
     INPUT:
 
@@ -784,7 +797,7 @@ def exchange_labels(G,i,j):
         return G
     u = min(i,j)
     v = max(i,j)
-    R = range(u) + [v] + range(u + 1,v) + [u] + range(v + 1,len(G.vertices()))
+    R = list(range(u)) + [v] + list(range(u + 1,v)) + [u] + list(range(v + 1,len(G.vertices())))
     return relabeling(G,R)
     
 
@@ -793,7 +806,7 @@ def normalize_vertex_names(G):
     Rename the vertices of this ``DiGraph`` so they are of the form `[0..n-1]`.
     
     ``G`` is expected to be a ``DiGraph`` with vertex set a set of integers (or at least
-    sortable elements). The output ``DiGraph`` is isomorphic to `G`, with vertices labeled
+    sortable elements). The output ``DiGraph`` is isomorphic to ``G``, with vertices labeled
     `[0..n-1]`, where the new vertex names respect the original order on vertex identifiers.
             
     INPUT:
@@ -823,224 +836,6 @@ def normalize_vertex_names(G):
 
 
 
-##################################################
-### Finding a spanning tree, rooted at vertex 0
-### along with the words labeling the paths in the
-### tree to each vertex
-##################################################
-
-def spanning_tree_and_paths(G,root=0):
-    r"""
-    Return a spanning tree `T` of this ``DiGraph``, a list of the leaves of `T`, and shortest paths in `T`, from the root to each vertex.
-    
-    ``G`` is expected to be a ``DiGraph`` with numerical edge labels. Computes a spanning tree `T`
-    (also a ``DiGraph``) by `\textit{breadth first search}` starting at vertex ``root`` --, along with a list of the non-root
-    leaves of `T`, and a dictionary associating with each vertex `v` the word labeling the geodesic path in `T`
-    from ``root`` to `v`.
-    
-    INPUT:
-
-        - ``G`` -- ``DiGraph``
-
-        - ``root`` -- a vertex of ``G``
-
-    OUTPUT: 
-
-        - a triple consisting of a ``DiGraph``, a list and a dictionary
-
-    EXAMPLES::
-    
-        sage: from stallings_graphs import FinitelyGeneratedSubgroup
-        sage: from stallings_graphs.about_automata import spanning_tree_and_paths
-        sage: L = [[3,1,-2,-1,3],[1,2,-1,-2,1,2],[1,2,-3,-3,1]]
-        sage: H = FinitelyGeneratedSubgroup.from_generators(L)
-        sage: G = H.stallings_graph()
-        sage: T,list_of_leaves,path_in_tree = spanning_tree_and_paths(G)
-        sage: T
-        Multi-digraph on 12 vertices
-        
-        ::
-        
-        sage: list_of_leaves
-        [2, 10, 3, 6, 5]
-        
-        ::
-        
-        sage: path_in_tree
-        {0: word: ,
-         1: word: 3,
-         2: word: 31,
-         3: word: -3,1,
-         4: word: -3,
-         5: word: 1,2,-1,
-         6: word: -2,-1,
-         7: word: -2,
-         8: word: 1,
-         9: word: 12,
-         10: word: -1,3,
-         11: word: -1}
-        
-    """
-    to_be_handled = []
-    to_be_handled.append(root)
-    visited_vertices = set()
-    visited_edges = []
-    list_of_leaves = []
-    path_in_tree = {}
-    path_in_tree[root] = Word()
-    while to_be_handled:
-        v = to_be_handled[0]
-        to_be_handled[:1] = []
-        visited_vertices.add(v)
-        is_leaf = True
-        if v == root:
-            is_leaf = False
-        for w in G.neighbors_out(v):
-            if w != v and w not in visited_vertices:
-                visited_vertices.add(w)
-                to_be_handled.append(w)
-                x = G.edge_label(v,w)[0]
-                visited_edges.append((v,w,x))
-                path_in_tree[w] = path_in_tree[v] + Word([x])
-                is_leaf = False
-        for w in G.neighbors_in(v):
-            if w != v and w not in visited_vertices:
-                visited_vertices.add(w)
-                to_be_handled.append(w)
-                x = G.edge_label(w,v)[0]
-                visited_edges.append((w,v,x))
-                path_in_tree[w] = path_in_tree[v] + Word([inverse_letter(x)])
-                is_leaf = False
-        if is_leaf:
-            list_of_leaves.append(v)
-    return DiGraph(visited_edges,multiedges = True), list_of_leaves, path_in_tree
-
-def basis_from_spanning_tree(G, T, D, root=0):
-    r"""
-    Return the basis (of the space of loops of ``G``at the ``root`` vertex) specified by the spanning tree ``T``.
-
-    ``G`` is expected to be a folded ``DiGraph`` with numerical edge labels. ``T`` (also a ``DiGraph``)
-    is expected to be a spanning tree of ``G``. ``D`` is expected to be a dictionary associating with each
-    vertex `v` of ``G`` (and ``T``) the word labeling the geodesic path in ``T``
-    from ``root`` to `v`. The output basis is a list of objects of class ``Word`` on a numerical alphabet, one
-    for each edge of ``G`` that is not in ``T``.
-    
-    INPUT:
-
-        - ``G`` -- ``DiGraph``
-
-        - ``T`` -- ``DiGraph``
-        
-        - ``D`` -- dictionary (the keys are the vertices of ``G`` and the values are of class ``Word``)
-
-        - ``root`` -- a vertex of ``G``
-
-    OUTPUT: 
-
-        - a list of objects of class ``Word`` (numerical)
-
-    EXAMPLES::
-    
-        sage: from stallings_graphs.about_automata import bouquet, spanning_tree_and_paths, basis_from_spanning_tree
-        sage: from stallings_graphs.about_folding import NT_fold
-        sage: generators = [[3,1,-2,-1,3],[1,2,-1,-2,1,2],[1,2,-3,-3,1]]
-        sage: G = NT_fold(bouquet(generators))
-        sage: T,L,D = spanning_tree_and_paths(G)
-        sage: basis_from_spanning_tree(G,T,D)
-        [word: -3,1,2,-1,-3, word: -2,-1,2,1,-2,-1, word: -1,3,3,-2,-1]
-
-        ::
-
-        sage: basis_from_spanning_tree(T,T,D)
-        []
-
-    """
-    basis = []
-    edge_set = set(G.edges()).difference(set(T.edges()))
-    for e in G.edges():
-        if e not in T.edges():
-            w = D[e[0]] + Word([e[2]]) + group_inverse(D[e[1]])
-            basis.append(w)
-    return basis
-
-def express_Word_in_basis_from_spanning_tree(G, T, w, root=0, alphabet_type = 'abc'):
-    r"""
-    Return the valuea word the basis (of the space of loops of ``G``at the ``root`` vertex) specified by the spanning tree ``T``.
-
-    ``G`` is expected to be a folded ``DiGraph`` with numerical edge labels. ``T`` (also a ``DiGraph``) is expected to be
-    a spanning tree of ``G``. ``w`` is expected to be a Word labeling a loop at ``root`` in ``G``. The output is a 
-    numerical Word which is the translation of ``w`` in the alphabet of the basis defined by ``T``.
-    
-    INPUT:
-
-        - ``G`` -- ``DiGraph``
-
-        - ``T`` -- ``DiGraph``
-        
-        - ``w`` -- ``Word``
-        
-        - ``root`` -- a vertex of ``G``
-
-        - ``alphabet_type`` -- string, which can be either ``'abc'`` or ``'123'``
-
-    OUTPUT: 
-
-        - ``Word``
-
-    EXAMPLES::
-    
-        sage: from stallings_graphs.about_automata import bouquet, spanning_tree_and_paths, basis_from_spanning_tree, express_Word_in_basis_from_spanning_tree
-        sage: from stallings_graphs.about_folding import NT_fold
-        sage: generators = ['abaa','ababb','ababab']
-        sage: G = NT_fold(bouquet(generators,alphabet_type = 'abc'))
-        sage: T,L,D = spanning_tree_and_paths(G)
-        sage: basis_from_spanning_tree(G,T,D)
-        [word: 1211, word: -1,2,2, word: -2,1,2]
-
-        ::
-
-        sage: w = Word('AbaabAABABab')
-        sage: express_Word_in_basis_from_spanning_tree(G,T,w)
-        word: 2,3,3,-1,3
-
-
-    """
-    if alphabet_type == 'abc':
-        u = translate_alphabetic_Word_to_numeric(w)
-    else:
-        u = w
-    PI = DiGraph_to_list_of_PartialInjection(G)
-    PIinv = [p.inverse_partial_injection() for p in PI]
-    translated = []
-    basis_list = []
-    for e in G.edges():
-        if e not in T.edges():
-            basis_list.append(e)
-    current_state = root
-    for a in u:
-        if a > 0:
-            mapping = PI[a-1]
-            next_state = mapping._list_of_images[current_state]
-            current_transition = (current_state,next_state,a)
-            if current_transition in basis_list:
-                next_letter = 1 + basis_list.index(current_transition)
-                translated.append(next_letter)
-        elif a < 0:
-            mapping = PIinv[-a-1]            
-            next_state = mapping._list_of_images[current_state]
-            current_transition = (next_state,current_state,-a)
-            if current_transition in basis_list:
-                next_letter = 1 + basis_list.index(current_transition)
-                translated.append(-next_letter)
-        elif a == 0:
-            raise ValueError('the 3rd argument is not a proper numerical Word (contains a non acceptable character')
-        current_state = next_state
-    if current_state != root:
-        raise ValueError('the 3rd argument does not label a loop at the root')
-    return Word(translated)
-
-
-
 
 ################################
 ### pruning, cyclically reducing
@@ -1073,22 +868,24 @@ def pruning(G):
         sage: GG
         Looped multi-digraph on 5 vertices
         
-        ::
+    ::
         
         sage: GGG = pruning(GG)
         sage: GGG
         Looped multi-digraph on 3 vertices
     
     """
+    from stallings_graphs.about_bases import spanning_tree_and_paths
     T,list_of_leaves,D = spanning_tree_and_paths(G, root = 0)
     if len(G.vertices()) > 1:
         while list_of_leaves:
             v = list_of_leaves.pop()
-            if len(G.neighbors_in(v)) + len(G.neighbors_out(v)) == 1:
+#            if len(G.neighbors_in(v)) + len(G.neighbors_out(v)) == 1:
+            if G.in_degree(v) + G.out_degree(v) == 1:
                 w = T.neighbors(v)[0]
                 G.delete_vertex(v)
                 T.delete_vertex(v)
-                if w != 0 and len(G.neighbors_in(w)) + len(G.neighbors_out(w)) == 1:
+                if w != 0 and T.in_degree(w) + T.out_degree(w) == 1:
                     list_of_leaves.append(w)
     GG = normalize_vertex_names(G)
     return GG
@@ -1097,7 +894,8 @@ def cyclic_reduction(G,trace=False):
     r"""
     Return the cyclic reduction of this ``DiGraph``.
     
-    ``G`` is expected to be ``DiGraph`` with numerical edge labels and vertex set of the form `[0..n]`.
+    ``G`` is expected to be a ``DiGraph`` with numerical edge labels and vertex set of the
+    form `[0..n]`.
     The cyclic reduction is obtained by iteratively deleting vertices of degree 1 (including the
     base vertex -- that is the difference with the ``prune`` method). Its vertex set is
     normalized to be of the form `[0..m]`. Note that the vertex labeled `0` in the cyclic
@@ -1112,13 +910,10 @@ def cyclic_reduction(G,trace=False):
     
     - ``trace``-- boolean
       
-    OUTPUT: if ``trace`` is set to ``False``:
+    OUTPUT: 
     
-    - ``Digraph``
-    
-    else:
-    
-    - a pair ``(GG, w)`` of a ``DiGraph`` and a ``Word``.
+    - a ``Digraph`` if ``trace`` is set to ``False``; and a pair ``(GG, w)`` of a ``DiGraph``
+      and a ``Word`` otherwise.
     
     EXAMPLES::
     
@@ -1130,13 +925,13 @@ def cyclic_reduction(G,trace=False):
         sage: G
         Looped multi-digraph on 6 vertices
         
-        ::
+    ::
         
         sage: G2 = cyclic_reduction(G)
         sage: G2
         Looped multi-digraph on 5 vertices
         
-        ::
+    ::
         
         sage: G2,w = cyclic_reduction(G,trace=True)
         sage: w
@@ -1168,7 +963,7 @@ def cyclic_reduction(G,trace=False):
 
 def fibered_product(G1,G2):
     r"""
-    Compute the fibered product (aka direct product) of two edge-labeled graphs.
+    Compute the fibered product (a.k.a. direct product) of two edge-labeled graphs.
     
     ``G1`` and ``G2`` are assumed to be of class ``DiGraph``, with edges labeled by positive integers.
     Their fibered product is the ``DiGraph`` whose vertex set is the Cartesian product of the sets of
@@ -1199,7 +994,7 @@ def fibered_product(G1,G2):
         sage: G12.vertices()
         [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2), (2, 0), (2, 1), (2, 2)]
         
-        ::
+    ::
         
         sage: len(G12.edges())
         26
