@@ -5,7 +5,7 @@ The methods for the class ``FinitelyGeneratedSubgroup`` use a number of ancillar
 A word is a string of characters from either a numerical or an alphabetical set
 of letters: ``alphabet_type='123'`` or ``'abc'``
 
-``alphabet_type='123'``: The positive letters form an interval `[1,r]`. Their inverses (aka
+``alphabet_type='123'``: The positive letters form an interval `[1,r]`. Their inverses (a.k.a.
 negative letters) are the corresponding negative integers. The symmetrized
 alphabet is the union of positive and negative letters (zero is NOT a letter).
 The `\textit{rank}` of a word is the maximal absolute value of a letter occurring in the word.
@@ -34,10 +34,14 @@ EXAMPLES::
     sage: group_inverse(w,alphabet_type='abc')
     word: abABBAbA
     
+::
+    
     sage: from stallings_graphs.about_words import free_group_reduction
     sage: w = Word([3,1,-2,-2,2,1,-1,2,5,-3])
     sage: free_group_reduction(w)
     word: 3,1,5,-3
+    
+::
 
     sage: from stallings_graphs.about_words import random_reduced_word
     sage: w = random_reduced_word(7,3)   #random
@@ -62,7 +66,7 @@ from sage.combinat.words.word import Word
 
 def positive_letters(r):
     r"""
-    Return the set of positive (numerical) letters up to `r`.
+    Return the list of positive (numerical) letters up to `r`.
     
     `r` is expected to be positive.
     
@@ -72,7 +76,7 @@ def positive_letters(r):
 
     OUTPUT:
 
-        - the list ``range(r)``
+        - list
         
     EXAMPLES::
 
@@ -81,7 +85,7 @@ def positive_letters(r):
         [1, 2, 3, 4, 5, 6]
 
     """
-    return range(1,r + 1)
+    return list(range(1,r + 1))
 
 def negative_letters(r):
     r"""
@@ -104,7 +108,7 @@ def negative_letters(r):
         [-1, -2, -3, -4, -5, -6]
 
     """
-    return range(-1,-r-1,-1)
+    return list(range(-1,-r-1,-1))
 
 def symmetric_alphabet(r):
     r"""
@@ -150,6 +154,9 @@ def inverse_letter(i):
         sage: from stallings_graphs.about_words import inverse_letter
         sage: inverse_letter(3)
         -3
+        
+    ::
+    
         sage: inverse_letter(-4)
         4
         
@@ -236,13 +243,16 @@ def translate_numeric_to_character(x):
         sage: from stallings_graphs.about_words import translate_numeric_to_character
         sage: translate_numeric_to_character(16)
         'p'
+        
+    ::
+    
         sage: translate_numeric_to_character(-10)
         'J'
     
     """
-    if x in range(-26,0):
+    if x in list(range(-26,0)):
         return chr(- x + 64)
-    elif x in range(1,27):
+    elif x in list(range(1,27)):
         return chr(x + 96)
     else:
         raise ValueError('the argument must be a non-zero integer between -26 and 26')
@@ -284,11 +294,11 @@ def translate_alphabetic_Word_to_numeric(w):
     
     INPUT:
 
-        - ``w`` -- Word
+        - ``w`` -- ``Word``
 
     OUTPUT:
 
-        - Word
+        - ``Word``
                     
     EXAMPLES::
 
@@ -296,6 +306,16 @@ def translate_alphabetic_Word_to_numeric(w):
         sage: translate_alphabetic_Word_to_numeric(Word('aBBaAc'))
         word: 1,-2,-2,1,-1,3
         
+    ::
+        
+        sage: translate_alphabetic_Word_to_numeric(Word(''))
+        word:
+        
+    ::
+        
+        sage: translate_alphabetic_Word_to_numeric(Word([]))
+        word:
+        
     """
     if len(w) == 0:
         return Word([])
@@ -311,11 +331,11 @@ def translate_numeric_Word_to_alphabetic(w):
     
     INPUT:
 
-        - ``w`` -- Word
+        - ``w`` -- ``Word``
 
     OUTPUT:
 
-        - Word
+        - ``Word``
     
     EXAMPLES::
 
@@ -353,12 +373,12 @@ def is_valid_Word(w, alphabet_type='123'):
         sage: is_valid_Word(w)
         True
         
-        ::
+    ::
         
         sage: is_valid_Word(Word('bABcac'), alphabet_type='abc')
         True
         
-        ::
+    ::
         
         sage: is_valid_Word(Word([2,-1,-2,0,1,3]))
         False
@@ -371,19 +391,19 @@ def is_valid_Word(w, alphabet_type='123'):
         return True
     if alphabet_type == '123':
         return all((x in ZZ and x != 0) for x in w)
-    L = range(65,91) + range(97,123)
+    L = list(range(65,91)) + list(range(97,123))
     return all(isinstance(x,str) for x in w) and all(ord(x) in L for x in w)
 
 def is_valid_list_of_Words(L,alphabet_type='123'):
     r"""
-    Return whether a list of ``Word``s is valid, in the sense of ``is_valid_Word``.
+    Return whether a list of ``Words`` is valid, in the sense of ``is_valid_Word``.
     
     `L` is expected to be a list of objects of class ``Word``. It is valid if all its components satisfy
     ``is_valid_Word``.
     
     INPUT:
 
-        - ``L`` -- ``List``
+        - ``L`` -- ``List`` of objects of the class ``Word``
 
         - ``alphabet_type`` -- string, which must be either ``'abc'`` or ``'123'``
 
@@ -397,6 +417,8 @@ def is_valid_list_of_Words(L,alphabet_type='123'):
         sage: L = [Word([2,-1,-2,3,1,3]), Word([1,2,-3,1,-1])]
         sage: is_valid_list_of_Words(L)
         True
+        
+    ::
         sage: L = [Word('bABcac'),'abcBA','baaCB']
         sage: is_valid_list_of_Words(L, alphabet_type='abc')
         True
@@ -407,6 +429,42 @@ def is_valid_list_of_Words(L,alphabet_type='123'):
     return all(is_valid_Word(w, alphabet_type) for w in L)
         
 
+def positive_alphabetic_content(w,check=False):
+    r"""
+    Return the set of positive letters which occur, or their inverse occurs in `w`.
+    
+    `w` is expected to be a ``Word`` on a numerical alphabet. If ``check`` is ``True``,
+    ``is_valid_Word(w,alphabet_type='123')`` is run.
+    
+    INPUT:
+
+        - ``w`` -- ``Word``
+        - ``check`` -- Boolean
+
+    OUTPUT:
+
+        - list of positive integers
+                
+    EXAMPLES::
+    
+        sage: from stallings_graphs.about_words import positive_alphabetic_content
+        sage: w = Word([3,1,-2,-2,5,-3])
+        sage: positive_alphabetic_content(w)
+        {1, 2, 3, 5}
+        
+    ::
+        
+        sage: w = Word([])
+        sage: positive_alphabetic_content(w)
+        set()
+        
+    """
+    if check == True:
+        if not(is_valid_Word(w,alphabet_type='123')):
+            raise ValueError('the argument is not a valid (numerical) Word: probably contains 0, which is not a letter, or is of type "abc".')
+            
+    return set(positive_value(x) for x in w.letters())
+
 def rank(w,check=False):
     r"""
     Return the least rank of a free group containing this ``Word``.
@@ -418,18 +476,25 @@ def rank(w,check=False):
     INPUT:
 
         - ``w`` -- ``Word``
-        - ``alphabet_type`` -- string, which must be either ``'abc'`` or ``'123'``
+        - ``check`` -- boolean
 
     OUTPUT:
 
         - integer
                 
-    EXAMPLES ::
+    EXAMPLES::
+    
         sage: from stallings_graphs.about_words import rank
         sage: w = Word([3,1,-2,-2,5,-3])
         sage: rank(w)
         5
         
+    ::
+        
+        sage: w = Word([])
+        sage: rank(w)
+        0
+        
     """
     if check == True:
         if not(is_valid_Word(w,alphabet_type='123')):
@@ -437,7 +502,7 @@ def rank(w,check=False):
     if len(w) == 0:
         return 0
     else:
-        return max([positive_value(x) for x in w])
+        return max(positive_alphabetic_content(w))
 
 ##############################################
 ## group properties: inverting, reducing, etc.
@@ -458,7 +523,7 @@ def group_inverse(w,alphabet_type='123',check=False):
 
     OUTPUT:
 
-        - Word
+        - ``Word``
             
     EXAMPLES ::
     
@@ -467,19 +532,19 @@ def group_inverse(w,alphabet_type='123',check=False):
         sage: group_inverse(w)
         word: -5,2,9,-1,-3
         
-        ::
+    ::
         
         sage: w = Word([-1,1,2,-2])
         sage: group_inverse(w)
         word: 2,-2,-1,1
         
-        ::
+    ::
         
         sage: w = Word([1])
         sage: group_inverse(w)
         word: -1
         
-        ::
+    ::
         
         sage: w = Word()
         sage: group_inverse(w)
@@ -525,7 +590,7 @@ def is_reduced(w,check=False):
         sage: is_reduced(w)
         True
         
-        ::
+    ::
         
         sage: u = Word([3,1,-2,2,5,-3])
         sage: is_reduced(u)
@@ -613,7 +678,7 @@ def is_cyclically_reduced(w,check=False):
         sage: is_cyclically_reduced(w)
         False
         
-        ::
+    ::
         
         sage: u = Word([3,1,-2,-2,5,3])
         sage: is_cyclically_reduced(u)
@@ -634,7 +699,7 @@ def cyclic_reduction_of_a_word(u):
     Return the elements of the cyclically reduced decomposition of this word.
     
     `u` is expected to be a ``Word`` on a numerical alphabet. The cyclically reduced
-    decomposition of `u` is the pair of Words `(v,w)` such that `v` is cyclically reduced,
+    decomposition of `u` is the pair of ``Words`` `(v,w)` such that `v` is cyclically reduced,
     and `u = w^{-1}vw`.
 
     INPUT:
