@@ -26,22 +26,27 @@ Methods implemented in this file:
 
 - ``eq``, to check whether two objects represent the same finitely generated subgroup
 
-- ``has_index``, to compute the index of the subgroup
-
 - ``basis``
 
-- ``intersection``
-
-- ``is_malnormal``, to check whether the subgroup is malnormal and, optionally, compute a witness of its non-malnormality
-
 - ``contains_element``, to check whether the subgroup contains a given word
 
 - ``contains_subgroup``, to check whether the subgroup contains a given subgroup
 
+- ``intersection``
+
+- ``has_index``, to compute the index of the subgroup
+
 - ``conjugated_by``, to compute the conjugate of a subgroup by a given word
 
 - ``is_conjugated_to``, to check whether two subgroups are conjugated and, optionally, compute a conjugating word
 
+- ``is_malnormal``, to check whether the subgroup is malnormal and, optionally, compute a witness of its non-malnormality
+
+- ``is_free_factor_of_ambient``, to check whether the subgroup is a free factor of the ambient group and, optionally, to compute a complement
+
+- ``is_free_factor_of_``, to check whether the subgroup is a free factor of another and, optionally, to compute a complement
+
+
 
 EXAMPLES::
 
@@ -51,14 +56,14 @@ EXAMPLES::
     sage: G
     A subgroup of the free group of rank 2, whose Stallings graph has 3 vertices
     
-    ::
+::
 
     sage: gens = [[1,2,5,-1,-2,2,1],[-1,-2,2,3],[1,2,3]]
     sage: G = FinitelyGeneratedSubgroup.from_generators(gens)
     sage: G
     A subgroup of the free group of rank 5, whose Stallings graph has 3 vertices
     
-    ::
+::
 
     sage: from stallings_graphs import FinitelyGeneratedSubgroup
     sage: from stallings_graphs.about_words import random_reduced_word
@@ -69,7 +74,7 @@ EXAMPLES::
     sage: H    # random
     A subgroup of the free group of rank 2, whose Stallings graph has 965 vertices
     
-    ::
+::
     
     sage: H = FinitelyGeneratedSubgroup.random_instance(15)
     sage: H
@@ -89,11 +94,12 @@ from sage.rings.integer_ring import ZZ
 from sage.graphs.digraph import DiGraph
 from sage.misc.cachefunc import cached_function
 from sage.misc.latex import LatexExpr
-#
-from partial_injections import PartialInjection
-from about_words import *
-from about_automata import *
-from about_folding import *
+
+from stallings_graphs.partial_injections import PartialInjection
+from stallings_graphs.about_words import *
+from stallings_graphs.about_automata import *
+from stallings_graphs.about_bases import *
+from stallings_graphs.about_folding import *
 # remplacer * par le nom des fonctions utilisées
 
 
@@ -102,14 +108,16 @@ class FinitelyGeneratedSubgroup(SageObject):
     Define the class ``FinitelyGeneratedSubgroup``, which represents subgroups of free groups.
     
     The representation of a finitely generated subgroup is by means of the partial
-    injections (on a set of the form `[0..(n-1)]`, one per generator of the ambient free group) which define its Stallings
-    graph, with base vertex 0. The Stallings graph of a subgroup is a uniquely defined finite directed graph,
+    injections (on a set of the form `[0..n-1]`, one per generator of the ambient free group)
+    which describes its Stallings graph, with base vertex 0. The Stallings graph of a subgroup is
+    a uniquely defined finite directed graph,
     whose edges are labeled by positive letters, rooted in a designated
     vertex, subject to three conditions: it must be connected, folded (no two edges
     with the same label share the same initial (resp. terminal) vertex),
     and every vertex must have valency 2 (in the underlying non-directed graph),
-    except possibly for the root (also known as base vertex). That is: a tuple of
-    partial injections.
+    except possibly for the root (also known as base vertex). That is: a subgroup is represented by
+    a tuple of partial injections on `[0..n-1]`, up to a relabeling of the elements of `[0..n-1]`
+    fixing the base vertex (namely 0).
 
     A ``FinitelyGeneratedSubgroup`` can be created from:
     
@@ -236,13 +244,10 @@ class FinitelyGeneratedSubgroup(SageObject):
         - ``ambient_rank`` -- integer, default value 2
         - ``verbose`` -- a boolean, default value False
 
-        OUTPUT: if ``verbose = False``
+        OUTPUT:
 
-        - an object of the class ``FinitelyGeneratedSubgroup``
-        
-        otherwise
-        
-        - a tuple of an object of the class ``FinitelyGeneratedSubgroup`` and an integer
+        - an object of the class ``FinitelyGeneratedSubgroup`` if ``verbose = False``,
+          and a tuple of an object of the class ``FinitelyGeneratedSubgroup`` and an integer otherwise
 
         EXAMPLES::
         
@@ -251,19 +256,19 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H
             A subgroup of the free group of rank 2, whose Stallings graph has 12 vertices
             
-            ::
+        ::
             
             sage: H = FinitelyGeneratedSubgroup.random_instance(2, ambient_rank = 0)
             sage: H
             A subgroup of the free group of rank 0, whose Stallings graph has 1 vertices
             
-            ::
+        ::
             
             sage: H,c = FinitelyGeneratedSubgroup.random_instance(12,3,verbose=True)
             sage: H
             A subgroup of the free group of rank 3, whose Stallings graph has 12 vertices
             
-            ::
+        ::
             
             sage: c #random
             1
@@ -276,9 +281,7 @@ class FinitelyGeneratedSubgroup(SageObject):
         whether they define a valid ``FinitelyGeneratedSubgroup``. If they do not, the tuple
         is tossed and another is drawn.
         
-        For a justification, see [BNW2008] F. Bassino, C. Nicaud, P. Weil. Random generation of
-        finitely generated subgroups of a free group, International Journal of Algebra and
-        Computation 18 (2008) 1-31.
+        For a justification, see [BNW2008]_.
         
 
         """
@@ -324,7 +327,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H
             A subgroup of the free group of rank 3, whose Stallings graph has 14 vertices
             
-            ::
+        ::
             
             sage: V = [0]
             sage: E = []
@@ -333,7 +336,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H
             A subgroup of the free group of rank 0, whose Stallings graph has 1 vertices
             
-            ::
+        ::
             
             sage: V = [0]
             sage: E = [(0,0,1)]
@@ -342,7 +345,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H
             A subgroup of the free group of rank 1, whose Stallings graph has 1 vertices
 
-            ::
+        ::
             
             sage: V = [0]
             sage: E = [(0,0,3)]
@@ -367,8 +370,8 @@ class FinitelyGeneratedSubgroup(SageObject):
         r"""
         Return the ``FinitelyGeneratedSubgroup`` specified by a set of generators.
         
-        ``generators`` is expected to be a list of valid ``Word`` objects, either numerical (on alphabet
-        `[1..r]` and inverse letters `[-r..-1]`) or alphabetical (`a`:`z` / `A`:`Z`).
+        ``generators`` is expected to be a list of valid ``Word`` objects, either numerical or
+        alphabetical, in accordanc with the value of ``alphabet_type``.
         The ``FinitelyGeneratedSubgroup`` produced represents the subgroup generated by these words.
         It is computed by operating a free group reduction on the elements of generators, computing the
         bouquet of these words and then creating the ``FinitelyGeneratedSubgroup`` specified by the bouquet.
@@ -389,21 +392,21 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H
             A subgroup of the free group of rank 2, whose Stallings graph has 3 vertices
             
-            ::
+        ::
             
             sage: gens = [[1,2,5,-1,-2,2,1],[-1,-2,2,3],[1,2,3]]
             sage: H = FinitelyGeneratedSubgroup.from_generators(gens)
             sage: H
             A subgroup of the free group of rank 5, whose Stallings graph has 3 vertices
             
-            ::
+        ::
             
             sage: L = []
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
             sage: H
             A subgroup of the free group of rank 0, whose Stallings graph has 1 vertices
             
-            ::
+        ::
             
             sage: L = [[2]]
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
@@ -412,7 +415,7 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         .. WARNING::
 
-            No exception will be raised if the input is not of the expected type.
+            No exception will be raised if the input is not of the expected type. Also:
             ``generators`` can be an empty list.
 
 
@@ -433,8 +436,8 @@ class FinitelyGeneratedSubgroup(SageObject):
         r"""
         Return the rank of the ambient free group of this ``FinitelyGeneratedSubgroup`` object.
         
-        The rank of the ambient free group is the number of partial injections which
-        specify this ``FinitelyGeneratedSubgroup``.
+        Exploits the fact that the rank of the ambient free group is the number of partial
+        injections which specify this ``FinitelyGeneratedSubgroup``.
         
         INPUT:
 
@@ -452,7 +455,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.ambient_group_rank()
             2
             
-            ::
+        ::
             
             sage: L = []
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
@@ -486,7 +489,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.stallings_graph_size()
             6
             
-            ::
+        ::
             
             sage: L = []
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
@@ -524,6 +527,8 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.rank()
             3
             
+        :
+            
             sage: L = []
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
             sage: H.rank()
@@ -590,7 +595,8 @@ class FinitelyGeneratedSubgroup(SageObject):
         
         Edge labels can be of the form `a_1,...,a_r` (``alphabet_type='123'``) or of the form
         `a,b,c,...,z` (``alphabet_type='abc'``). The visualization tool can be ``graph.plot``
-        (with a color coding for the base vertex) or Sébastien Labbé's ``TikzPicture`` `method <http://www.slabbe.org/Sage>`_. 
+        (with a color coding for the base vertex) or Sébastien Labbé's ``TikzPicture``
+        `method <http://www.slabbe.org/Sage>`_. 
 
         INPUT:
 
@@ -600,14 +606,11 @@ class FinitelyGeneratedSubgroup(SageObject):
         
         - ``visu_tool`` -- a string which is either ``'plot'`` or ``'tikz'``
         
-        OUTPUT: if ``visu_tool`` is set to ``'plot'``
+        OUTPUT:
 
-        - a visualization of the Stallings graph using ``graph.plot``
-        
-        if ``visu_tool`` is set to ``'tikz'``
-        
-        - a visualization of the Stallings graph using ``TikzPicture``. In this case, the output can be saved as a ``.png``, ``.pdf`` or ``.tex`` file.
-        
+        - a visualization of the Stallings graph using ``graph.plot`` or using ``TikzPicture``,
+          according to the value of ``visu_tool``. In the ``'tikz'`` case, the output can be saved
+          as a ``.png``, ``.pdf`` or ``.tex`` file        
 
         EXAMPLES ::
         
@@ -617,7 +620,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.show_Stallings_graph(alphabet_type='abc',visu_tool='plot')
             Graphics object consisting of 28 graphics primitives
             
-            ::
+        ::
             
             sage: t = H.show_Stallings_graph(alphabet_type='abc',visu_tool='tikz')
             sage: # one can then type t.png, t.tex, t.pdf
@@ -648,7 +651,7 @@ class FinitelyGeneratedSubgroup(SageObject):
         
         OUTPUT:
         
-        - boolean
+        - a boolean if ``verbose`` is set to ``False``; a pair of a boolean and a string otherwise
 
         EXAMPLES::
         
@@ -658,7 +661,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.is_valid()
             True
             
-            ::
+        ::
             
             sage: M = [PartialInjection([1,2,None,4,5,3]), PartialInjection([0,2,1,None,4,3])]
             sage: K = FinitelyGeneratedSubgroup(M)
@@ -680,21 +683,21 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         """
         
-        from partial_injections import is_valid_partial_injection
+        from stallings_graphs.partial_injections import is_valid_partial_injection
         b1 = all(is_valid_partial_injection(p._list_of_images) for p in self._partial_injections)
         if verbose:
             if not b1:
-                print 'not all given partial injections are truly partial injections'
+                print('not all given partial injections are truly partial injections')
             else:
-                print 'all given partial injections are okay'
+                print('all given partial injections are okay')
         if b1:        
             G = self.stallings_graph()
             b1 = b1 and G.is_connected()
             if verbose:
                 if not b1:
-                    print 'the resulting graph is not connected'
+                    print('the resulting graph is not connected')
                 else:
-                    print 'the graph is connected'
+                    print('the graph is connected')
         if b1:
             b2 = True
             for v in G.vertices():
@@ -704,9 +707,9 @@ class FinitelyGeneratedSubgroup(SageObject):
             b1 = b1 and b2
             if verbose:
                 if not b1:
-                    print 'vertex {} has degree less than 2'.format(v_exceptional)
+                    print('vertex {} has degree less than 2'.format(v_exceptional))
                 else:
-                    print 'all vertices other than the base vertex (#0) have degree at least 2'
+                    print('all vertices other than the base vertex (#0) have degree at least 2')
         return b1
     
     
@@ -766,7 +769,7 @@ class FinitelyGeneratedSubgroup(SageObject):
         """
         b = isinstance(other, FinitelyGeneratedSubgroup)
         if not b:
-            print 'the second argument is not a FinitelyGeneratedSubgroup'
+            print('the second argument is not a FinitelyGeneratedSubgroup')
             return False
         else:
             G = self.stallings_graph()
@@ -783,7 +786,9 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         - ``self`` -- a ``FinitelyGeneratedSubgroup``
         
-        OUTPUT: an integer or ``+Infinity``
+        OUTPUT:
+        
+        - an integer or ``+Infinity``
         
     
         EXAMPLES::
@@ -794,15 +799,15 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.has_index()
             +Infinity
             
-            ::
+        ::
             
             sage: H = FinitelyGeneratedSubgroup([])
             sage: H.has_index()
             1
             
-            ::
+        ::
             
-            sage: HH = FinitelyGeneratedSubgroup.from_generators(['ab', 'ba', 'Abab'],alphabet_type = 'abc')
+            sage: HH = FinitelyGeneratedSubgroup.from_generators(['ab', 'ba', 'Abab'], alphabet_type = 'abc')
             sage: HH.has_index()
             2
             
@@ -814,11 +819,11 @@ class FinitelyGeneratedSubgroup(SageObject):
             from sage.rings.infinity import Infinity as oo
             return oo
 
-    def basis(self, alphabet_type = '123'):
+    def basis(self, alphabet_type = 'abc'):
         r"""
         Return a basis of this subgroup.
         
-        The input semigroup is expected to be an object of the class ``FinitelyGeneratedSubgroup``.
+        The input is expected to be an object of the class ``FinitelyGeneratedSubgroup``.
         The variable ``alphabet_type`` determines whether the words in the output
         are numerical or alphabetic.
         
@@ -835,27 +840,32 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: from stallings_graphs import FinitelyGeneratedSubgroup
             sage: L = [[3,1,-2,-1,3],[1,2,-1,-2,1,2],[1,2,-3,-3,1]]
             sage: H = FinitelyGeneratedSubgroup.from_generators(L)
-            sage: H.basis()
+            sage: H.basis(alphabet_type = '123')
             [word: -3,1,2,-1,-3, word: -2,-1,2,1,-2,-1, word: -1,3,3,-2,-1]
             
-            ::
+        ::
+            
+            sage: H.basis()
+            [word: CabAC, word: BAbaBA, word: AccBA]
+            
+        ::
             
             sage: H = FinitelyGeneratedSubgroup([])
             sage: H.basis()
             []
             
-            ::
+        ::
             
-            sage: H = FinitelyGeneratedSubgroup.from_generators(['a'],alphabet_type = 'abc')
-            sage: H.basis(alphabet_type = 'abc')
+            sage: H = FinitelyGeneratedSubgroup.from_generators(['A'],alphabet_type = 'abc')
+            sage: H.basis()
             [word: a]
             
         """
         G = self.stallings_graph()
         T,L,D = spanning_tree_and_paths(G)
-        B = basis_from_spanning_tree(G, T, D)
-        if alphabet_type == 'abc':
-            B = [translate_numeric_Word_to_alphabetic(w) for w in B]
+        B = basis_from_spanning_tree(G, T, D,alphabet_type=alphabet_type)
+#        if alphabet_type == 'abc':
+#            B = [translate_numeric_Word_to_alphabetic(w) for w in B]
         return B
     
     def intersection(self, K):
@@ -886,7 +896,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.intersection(K)
             A subgroup of the free group of rank 4, whose Stallings graph has 1 vertices
             
-            ::
+        ::
             
             sage: L = ['ab', 'aaBa', 'bbAb']
             sage: H = FinitelyGeneratedSubgroup.from_generators(L, alphabet_type = 'abc')
@@ -894,7 +904,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: K = FinitelyGeneratedSubgroup.from_generators(M, alphabet_type = 'abc')
             sage: S = H.intersection(K)
             sage: S.basis()
-            [word: 2121, word: 2,1,-2,1,-2,-2, word: 12, word: -1,2,-1,-1]
+            [word: baba, word: baBaBB, word: ab, word: AbAA]
     
         """
         r = max(self.ambient_group_rank(),K.ambient_group_rank())
@@ -927,15 +937,13 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         - ``self`` -- ``FinitelyGeneratedSubgroup``
         - ``alphabet_type`` -- a string which is either ``'abc'`` or ``'123'``
-        - ``witness``-- a boolean
+        - ``witness`` -- a boolean
         
-        OUTPUT: if ``witness`` is set to ``False``
+        OUTPUT: 
         
-        - a boolean
-        
-        if ``witness`` is set to ``True``
-        
-        - a tuple of the form ``(True, None, None)`` if this subgroup is malnormal, and of the form ``(False,s,t)`` if it is not, where `s` and `t` are of the class ``Word``.
+        - a boolean if ``witness`` is set to ``False``; and if ``witness`` is set to ``True``, then
+          a tuple of the form ``(True, None, None)`` if the subgroup is malnormal, and of the form
+          ``(False,s,t)`` if it is not, where `s` and `t` are of the class ``Word``.
 
         EXAMPLES ::
 
@@ -992,8 +1000,8 @@ class FinitelyGeneratedSubgroup(SageObject):
         G = self.stallings_graph()
         GG = fibered_product(G,G)
         CC = GG.connected_components()
-        # Let baseCC be the connected component containing vertex (0,0) (isomorphic to G).
-        # The E number of edges of GG outside baseCC is len(GG.edges()) - len(G.edges())
+        # Let baseCC be the connected component containing vertex (0,0): it is isomorphic to G.
+        # The number E of edges of GG outside baseCC is len(GG.edges()) - len(G.edges())
         # and the corresponding number of vertices V is len(GG.vertices()) - len(G.vertices()).
         # The connected components of GG other than baseCC are all trees if and only if
         # V - E = number of (non-base) connected components.
@@ -1011,15 +1019,15 @@ class FinitelyGeneratedSubgroup(SageObject):
         look_for_non_trivial_CC = True
         i = 0
         while look_for_non_trivial_CC:
-            X = CC[i]
-            GGX = GG.subgraph(vertices=X)
-            if len(GGX.edges()) > len(X) - 1:
+            GGX = GG.subgraph(vertices=CC[i])
+            if len(GGX.edges()) > len(CC[i]) - 1:
                 look_for_non_trivial_CC = False
             else:
                 i += 1
+        X = CC[i]
         # now X is an interesting connected component
         T,L,P = spanning_tree_and_paths(GGX,root=X[0])
-        E = GGX.edges()
+        E = list(GGX.edges())
         F = T.edges()
         for e in F:
             E.remove(e)
@@ -1116,10 +1124,10 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         ::
 
-            sage: w = Word('abba')
-            sage: K = FinitelyGeneratedSubgroup.from_generators(['abb'], alphabet_type = 'abc')
+            sage: LL = ['abba','bAbA']
+            sage: K = FinitelyGeneratedSubgroup.from_generators(LL, alphabet_type = 'abc')
             sage: H.contains_subgroup(K)
-            False
+            True
 
         ::
 
@@ -1127,13 +1135,13 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.contains_subgroup(K)
             False
             
-            ::
+        ::
 
             sage: K.contains_subgroup(H)
             True
 
         """
-        B = other.basis()
+        B = other.basis(alphabet_type='123')
         return all(self.contains_element(b) for b in B)
 
     def conjugated_by(self, w, alphabet_type = '123'):
@@ -1164,11 +1172,6 @@ class FinitelyGeneratedSubgroup(SageObject):
             
         ::
         
-            sage: w = Word('abBA')
-            sage: K = H.conjugated_by(w, alphabet_type='abc')
-            sage: H == K
-            True
-        
             sage: w1 = Word('bA')
             sage: K1 = H.conjugated_by(w1, alphabet_type='abc')
             sage: K1
@@ -1180,6 +1183,15 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: K2 = H.conjugated_by(w2, alphabet_type='abc')
             sage: K2
             A subgroup of the free group of rank 2, whose Stallings graph has 5 vertices
+            
+        ::
+        
+            sage: w = Word('abba')
+            sage: K3 = H.conjugated_by(w, alphabet_type='abc')
+            sage: H == K3
+            True
+        
+
 
 
         """
@@ -1218,15 +1230,14 @@ class FinitelyGeneratedSubgroup(SageObject):
 
         INPUT:
 
-        - ``other``-- ``FinitelyGeneratedSubgroup``
-        - ``conjugator``-- boolean
+        - ``other`` -- ``FinitelyGeneratedSubgroup``
+        - ``conjugator`` -- boolean
         - ``alphabet_type`` -- a string which can be either ``'abc'`` or ``'123'``
 
         OUTPUT: 
         
-        - a boolean
-        
-        - or, if ``conjugator`` is ``True``, a tuple consisting of a boolean and a ``Word`` or ``None``.
+        - a boolean or, if ``conjugator`` is ``True``, a tuple consisting of a boolean and a
+          ``Word`` or ``None``.
 
         EXAMPLES::
 
@@ -1238,7 +1249,7 @@ class FinitelyGeneratedSubgroup(SageObject):
             sage: H.is_conjugated_to(K)
             True
             
-            ::
+        ::
             
             sage: b,w = H.is_conjugated_to(K,conjugator=True,alphabet_type = 'abc')
             sage: w
@@ -1279,14 +1290,242 @@ class FinitelyGeneratedSubgroup(SageObject):
                     return True
         return (False, None) if conjugator else False
 
+    def SW_is_free_factor_of_ambient(self, complement = True, alphabet_type = '123'):
+        r"""
+        Return whether ``self`` is a free factor of the ambient group and, if ``complement`` is
+        set to ``True``, gives either a statement about it not being a free factor, or if it is,
+        a basis of a complement of ``self`` (in numerical or in alphabetic form depending on
+        ``alphabet_type``).
 
+        INPUT:
+
+        - ``complement``-- boolean
+        - ``alphabet_type`` -- a string which can be either ``'abc'`` or ``'123'``
+
+        OUTPUT: 
+        
+        - a boolean if ``complement`` is ``False`` and a pair of a boolean and either a string of a list of objects of type ``Word`` otherwise
+
+        EXAMPLES::
+
+            sage: from stallings_graphs import FinitelyGeneratedSubgroup
+            sage: L1 = ['ac','bacd','ed']
+            sage: H1 = FinitelyGeneratedSubgroup.from_generators(L1, alphabet_type='abc')
+            sage: H1.SW_is_free_factor_of_ambient(complement = True, alphabet_type = 'abc')
+            (True, [word: baE, word: B])
+
+        ::
+
+            sage: L2 = ['acac','bacd','ed']
+            sage: H2 = FinitelyGeneratedSubgroup.from_generators(L2, alphabet_type='abc')
+            sage: H2.SW_is_free_factor_of_ambient(complement = True, alphabet_type='abc')
+            (False, 'the 1st argument is not a free factor of the second')
+
+        ::
+
+            sage: H = FinitelyGeneratedSubgroup.from_generators(['A','d'], alphabet_type='abc')
+            sage: H.SW_is_free_factor_of_ambient(complement = True, alphabet_type='abc')
+            (True, [word: b, word: c])
+        
+        ALGORITHM:
     
-
-                    
-
-
+            The algorithm implemented is from [SW2008]_. Be aware that the worst-case complexity
+            is polynomial in the size of the argument subgroup but exponential in the rank
+            difference between that subgroup and the ambient group.
+        """
+        from stallings_graphs.about_free_factors import SilvaWeil_free_factor_of_ambient
+        if complement:
+            (valeur,base) = SilvaWeil_free_factor_of_ambient(self, maxletter = 0, complement = True)
+            if alphabet_type == 'abc' and valeur == True:
+                base = [translate_numeric_Word_to_alphabetic(w) for w in base]
+            return (valeur,base)
+        else:
+            return SilvaWeil_free_factor_of_ambient(self, maxletter = 0, complement = False)
     
     
+    def SW_is_free_factor_of(self, other, complement = True, alphabet_type = '123'):
+        r"""
+        Return whether ``self`` is a free factor of ``other`` and, if ``complement`` is
+        set to ``True``, gives either a statement about it not being a free factor, or if it is,
+        a basis of a complement of ``self`` in ``other`` (in numerical or in alphabetic form
+        depending on ``alphabet_type``).
+        
+        ``other`` is expected to be a ``FinitelyGeneratedSubgroup``
 
+        INPUT:
+
+        - ``other`` -- ``FinitelyGeneratedSubgroup``
+        - ``complement`` -- boolean
+        - ``alphabet_type`` -- a string which can be either ``'abc'`` or ``'123'``
+
+        OUTPUT: 
+        
+        - a boolean if ``complement`` is ``False``, and a pair of a boolean and either a string or a list of objects of type ``Word`` otherwise
+
+        EXAMPLES::
+
+            sage: from stallings_graphs import FinitelyGeneratedSubgroup
+            sage: LH = [[2,-3,1,3,2,3,-2,-1,2,-3,-1], [3,1,1,1,-3,-1], [1,3,-2,-1,2,-1,2], [3,2,3,-1,2,-1]]
+            sage: LK = [[2,-3], [1,1], [1,3,-2,1,2,-3,-1], [3,2], [3,1,-3,-1], [1,3,2,-1], [1,3,3,-1], [1,3,1,-3]]
+            sage: H = FinitelyGeneratedSubgroup.from_generators(LH, alphabet_type='123')
+            sage: K = FinitelyGeneratedSubgroup.from_generators(LK, alphabet_type='123')
+            sage: H.SW_is_free_factor_of(K, complement = True)
+            (True, [word: 3,2,3,1,2,-1, word: 32, word: 3,1,3,-1, word: 11])
+        
+        ::
+        
+            sage: H.SW_is_free_factor_of(K, complement = False)
+            True
+        
+        ::
+        
+            sage: LH = [[-3,1,-2,-1,-1,-3,2,2,3], [-3,-1,-1,3,1,1,-3,-1,3,1,3,3], [-3,1,3,-1,-1,-3,1,1,1,3,-1,-1], [1,1,-3,1,3,1,1,-3,-1,3]]
+            sage: LK = [[1,1,2,-1,3], [1,1,3,-1], [-3,1,3,-1,-1], [-3,1,1,3], [-3,2,3], [1,3,3]]
+            sage: H = FinitelyGeneratedSubgroup.from_generators(LH, alphabet_type='123')
+            sage: K = FinitelyGeneratedSubgroup.from_generators(LK, alphabet_type='123')
+            sage: H.SW_is_free_factor_of(K, complement = True)
+            (False, 'the 1st argument is not a free factor of the second')
+        
+        ::
+        
+            sage: H.SW_is_free_factor_of(K, complement = False)
+            False
+        
+        ::
+        
+            sage: LH = [[3,1,-2,-1,-1,-3,2,2,3], [-3,-1,-1,3,1,1,-3,-1,3,1,3,3], [-3,1,3,-1,-1,-3,1,1,1,3,-1,-1], [1,1,-3,1,3,1,1,-3,-1,3]]
+            sage: LK = [[1,1,2,-1,3], [1,1,3,-1], [-3,1,3,-1,-1], [-3,1,1,3], [-3,2,3], [1,3,3]]
+            sage: H = FinitelyGeneratedSubgroup.from_generators(LH, alphabet_type='123')
+            sage: K = FinitelyGeneratedSubgroup.from_generators(LK, alphabet_type='123')
+            sage: H.SW_is_free_factor_of(K, complement = True)
+            (False, '1st argument not contained in 2nd')
+        
+        ::
+        
+            sage: H.SW_is_free_factor_of(K, complement = False)
+            False
+        
+        ::
+        
+            sage: H = FinitelyGeneratedSubgroup.from_generators(['bba','bAbaB'], alphabet_type='abc')
+            sage: K = FinitelyGeneratedSubgroup.from_generators(['a', 'bb', 'bAbaB'], alphabet_type='abc')
+            sage: H.SW_is_free_factor_of(K, complement = True, alphabet_type = 'abc')
+            (True, [word: BB])
+        
+        ::
+        
+            sage: H.SW_is_free_factor_of(K, complement = False, alphabet_type = 'abc')
+            True
+        
+        ::
+        
+            sage: H = FinitelyGeneratedSubgroup.from_generators(['a','B'], alphabet_type='abc')
+            sage: K = FinitelyGeneratedSubgroup.from_generators(['a','b','d'], alphabet_type='abc')
+            sage: H.SW_is_free_factor_of(K, complement = True, alphabet_type = 'abc')
+            (True, [word: d])
+        
+        ::
+        
+            sage: H.SW_is_free_factor_of(K, complement = False, alphabet_type = 'abc')
+            True
+
+        
+        ALGORITHM:
     
+            The algorithm implemented is from [SW2008]_. Be aware that the worst-case complexity
+            is polynomial in the size of the two argument subgroups, but exponential in the
+            difference between their ranks.
+        """
+
+        from stallings_graphs.about_free_factors import SilvaWeil_free_factor_of
+        if complement:
+            (valeur,base) = SilvaWeil_free_factor_of(self, other, complement = True)
+            if alphabet_type == 'abc' and valeur == True:
+                base = [translate_numeric_Word_to_alphabetic(w) for w in base]
+            return (valeur,base)
+        else:
+            return SilvaWeil_free_factor_of(self, other, complement = False)
+
+
+    def algebraic_extensions(self):
+        r"""
+        Return a dictionary listing the algebraic extensions of ``self``. The keys are integers
+        without any particular meaning, except key 0 corresponds to ``H`` itself. The entries are
+        lists of an algebraic extension, sets of keys corresponding to parents and children
+        of this extension (not a Hasse diagram of the containment relation, but including such
+        a diagram), and a boolean indicating whether the extension is e-algebraic.
+        
+        For a definition of algebraic and e-algebraic extensions, see [MVW2007]_.
+        
+        INPUT:
+
+        - ``self`` -- an object of the class ``FinitelyGeneratedSubgroup``.
+
+        OUTPUT: 
+        
+        - a dictionary whose keys are integers and whose entries are lists of an object of type
+          ``FinitelyGeneratedSubgroup``, two sets of keys, and a boolean
+
+        EXAMPLES::
+
+            sage: from stallings_graphs import FinitelyGeneratedSubgroup
+            sage: from stallings_graphs.about_free_factors import compute_algebraic_extensions
+            sage: testgens = ['aba','bab']
+            sage: testH = FinitelyGeneratedSubgroup.from_generators(testgens,alphabet_type='abc')
+            sage: testH.algebraic_extensions()
+            {0: [A subgroup of the free group of rank 2, whose Stallings graph has 5 vertices,
+              set(),
+              {1},
+              True],
+             1: [A subgroup of the free group of rank 2, whose Stallings graph has 1 vertices,
+              {0},
+              set(),
+              True]}
+
+        ::
+
+            sage: testgens = ['ab','cd']
+            sage: testH = FinitelyGeneratedSubgroup.from_generators(testgens,alphabet_type='abc')
+            sage: testH.algebraic_extensions()
+            {0: [A subgroup of the free group of rank 4, whose Stallings graph has 3 vertices,
+              set(),
+              set(),
+              True]}
+
+        ::
+
+            sage: testgens = ['ABBaaBABa','Baba','Abababba','AbabbABa','ABabAba']
+            sage: testH = FinitelyGeneratedSubgroup.from_generators(testgens,alphabet_type='abc')
+            sage: testH.algebraic_extensions()
+            {0: [A subgroup of the free group of rank 2, whose Stallings graph has 10 vertices,
+              set(),
+              {3, 6, 11},
+              True],
+             3: [A subgroup of the free group of rank 2, whose Stallings graph has 1 vertices,
+              {0, 6, 11},
+              set(),
+              True],
+             6: [A subgroup of the free group of rank 2, whose Stallings graph has 3 vertices,
+              {0, 11},
+              {3},
+              True],
+             11: [A subgroup of the free group of rank 2, whose Stallings graph has 8 vertices,
+              {0},
+              {3, 6},
+              True]}
+
+        """
+
+        original_basis = self.basis(alphabet_type='123')
+        from stallings_graphs.about_free_factors import compute_algebraic_extensions
+        semilattice_AE = compute_algebraic_extensions(self)
+        semilattice = {}
+        for key in semilattice_AE.keys():
+            newgenerators = original_basis + semilattice_AE[key][2]
+            newextension = FinitelyGeneratedSubgroup.from_generators(newgenerators, alphabet_type='123')
+            semilattice[key] = [newextension, semilattice_AE[key][0], semilattice_AE[key][1], semilattice_AE[key][4]]
+        return semilattice
+                                
+            
+
 
